﻿using System.Data;
using AutoMapper;
using Microsoft.Data.Sqlite;
using Microsoft.EntityFrameworkCore;
using Moq;
using Samples.DLL;
using Xunit;

namespace Samples.BLL.Tests
{
    public class TestServiceTests
    {
        [Fact(DisplayName = "数据驱动测试--不依赖外部数据")]
        public async Task GetBalanceAsyncTest()
        {
            var mock = new Mock<IUserService>();
            mock.Setup(w => w.Get(12321)).Returns(new UserIdentity { UserId = 1, Name = "MockName1" });
            mock.Setup(w => w.Get(321)).Returns(new UserIdentity { UserId = 1000, Name = "MockName2" });

            var service = new TestService(mock.Object);

            Assert.True(await service.GetBalanceAsync(321) == 10000);
            Assert.True(await service.GetBalanceAsync(12321) == 200);
        }

        [Theory(DisplayName = "数据驱动测试--外部传入数据")]
        [InlineData(100, 200)]
        [InlineData(2345, 10000)]
        public async Task GetBalanceAsync_Theory_Test(long userId, decimal result)
        {
            var mock = new Mock<IUserService>();
            mock.Setup(w => w.Get(userId)).Returns(new UserIdentity { UserId = userId, Name = "MockName2" });

            var service = new TestService(mock.Object);

            Assert.True(await service.GetBalanceAsync(userId) == result);
        }

        [Theory(DisplayName = "数据驱动测试--外部Excel数据")]
        [ExcelData("Data\\ExcelDataSource.xlsx", "TestData")]
        public async Task GetBalanceAsync_Excel_Test(int id, string 主诊断编码, string 主诊断名称, string 手术编码, string 手术名称, string 实际入组, string 期望入组)
        {
            var mock = new Mock<IUserService>();
            mock.Setup(w => w.Get(0)).Returns(new UserIdentity { UserId = 0, Name = "MockName2" });

            var service = new TestService(mock.Object);

            //Assert.True(await service.JoinGroup(主诊断编码, 手术编码, 实际入组, 期望入组) == 期望入组);
            Assert.True(await service.JoinGroup(主诊断编码, 手术编码, 实际入组, 期望入组) == 实际入组);
        }

        [Fact(DisplayName = "数据驱动测试--外部 MEMORY 数据")]
        public async Task GetBalanceAsync_Inmemory_Test()
        {
            var _connection = new SqliteConnection("Filename=:memory:");
            if (_connection.State != ConnectionState.Open) _connection.Open();
            var _contextOptions = new DbContextOptionsBuilder<TestDbContext>().UseSqlite(_connection).Options;

            using var context = new TestDbContext(_contextOptions);
            context.Database.EnsureCreated();

            Init<ExcelData>(context, sheetName: "TestData").Wait();
            context.SaveChanges();


            var mock = new Mock<IUserService>();
            mock.Setup(w => w.Get(0)).Returns(new UserIdentity { UserId = 0, Name = "MockName2" });

            var service = new TestService(mock.Object);

            foreach (var item in context.ExcelData.ToList())
            {
                Assert.True(await service.JoinGroup(item.主诊断编码, item.手术编码, item.实际入组, item.期望入组) == item.实际入组);
            }
        }
        private async Task Init<TEntity>(DbContext context, string sheetName) where TEntity : class
        {
            var path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Data", "ExcelDataSource.xlsx");
            var datas = ExcelHelper.GetData(path, sheetName);

            var config = new MapperConfiguration(cfg => { });
            var mapper = config.CreateMapper();

            var objs = mapper.Map<List<TEntity>>(datas);
            await context.Set<TEntity>().AddRangeAsync(objs);
        }
    }
}